@@ -1,6 +1,5 @@
 """
 Tests for DRUHG clustering algorithm
-Shamelessly based on (i.e. ripped off from) the HDBSCAN test code ))
 """
 import pickle
 import numpy as np
@@ -8,7 +7,7 @@ import pandas as pd
 from scipy.spatial import distance
 from scipy import sparse
 from scipy import stats
-import pytest
+import pytest  #delete __init__.py or it wont work
 
 # from sklearn.utils.estimator_checks import check_estimator
 # from sklearn.utils.testing import (assert_equal,
@@ -44,7 +43,9 @@ X = np.vstack([moons, blobs])
 
 _plot_graph = 0
 
-def test_iris():
+def test_iris(filename=None):
+    if filename is None:
+        filename = test_iris.__name__
     iris = datasets.load_iris()
     XX = iris['data']
     # print (XX, type(XX))
@@ -54,7 +55,7 @@ def test_iris():
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_iris1.png')
+        plt.savefig(filename+'1'+'.png')
 
     ari = adjusted_rand_score(iris['target'], labels)
     print ('iris ari', ari)
@@ -65,7 +66,7 @@ def test_iris():
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_iris2.png')
+        plt.savefig(filename + '2' + '.png')
 
     ari = adjusted_rand_score(iris['target'], labels)
     print ('iris ari', ari)
@@ -78,14 +79,17 @@ def test_plot_mst():
     dr.fit(XX)
     dr.minimum_spanning_tree_.plot()
 
-def test_plot_dendrogram():
+def test_plot_dendrogram(filename=None):
+    if filename is None:
+        filename = test_plot_dendrogram.__name__
     iris = datasets.load_iris()
     XX = iris['data']
     dr = DRUHG(max_ranking=50, limit2=int(len(XX)/2),fix_outliers=1) #, limit1=0, limit2=int(len(XX)/2), fix_outliers=1)
     dr.fit(XX)
-    # plt.close('all')
-    dr.single_linkage_.plot()
-    # plt.savefig('test_square.png')
+    if _plot_graph:
+        plt.close('all')
+        dr.single_linkage_.plot()
+        plt.savefig(filename+'.png')
 
 def test_plot_one_dimension():
     iris = datasets.load_iris()
@@ -122,30 +126,69 @@ def test_2and3():
     assert (labels[2] == labels[3] == labels[4])
     assert (labels[0] != labels[2])
     assert (n_clusters == 2)
-    # assert (1==0)
+    # assert (False)
 
-def test_line():
-    XX = [[0.,1.],[0.,2.],[0.,3.],[0.,4.]]
+def test_scaled_flatrangle(filename=None):
+    if filename is None:
+        filename = test_scaled_flatrangle.__name__
+    for i in range(-5, 4):
+        test_line(size=3, scale=10 ** i, filename=filename, all_clusters=True)
+
+def test_rightangle(scale = 1., filename = None, all_clusters = True):
+    if filename is None:
+        filename = test_rightangle.__name__
+    XX = [[0., 0.], [scale, 0], [0, scale]]
     XX = np.array(XX)
-    dr = DRUHG(max_ranking=200, limit1=1, verbose=False)
+    dr = DRUHG(algorithm='slow', max_ranking=200, limit1=1, limit2=1000, verbose=False)
     dr.fit(XX)
-    # starts from the middle
-    print (dr.mst_[0], dr.mst_[1])
-    print (dr.labels_)
-    assert (dr.mst_[0]*dr.mst_[1]==2)
-    # zero clusters cause it always grows by 1
-    print ('pairs', dr.mst_)
-    print ('labels', dr.labels_)
+    # two clusters
+    # assert (len(dr.parents_) == 2)
+
     labels = dr.labels_
-    assert (not all(x == labels[0] for x in labels))
-    assert (labels[0] == labels[3])
-    assert (labels[1] == labels[2])
-    # assert (1==0)
-# #
-def test_longline():
+    print('pairs', dr.mst_)
+    print('labels', dr.labels_)
+    n_clusters = len(set(labels)) - int(-1 in labels)
+    print('n_clusters', n_clusters)
+
+    assert (all(x == labels[0] for x in labels))
+
+def test_scaled_rightangle(scale = 1., filename = None, all_clusters = True):
+    if filename is None:
+        filename = test_scaled_rightangle.__name__
+    for i in range(-5, 4):
+        test_rightangle(scale=10 ** i, filename=filename, all_clusters=True)
+
+def test_equilateral(scale = 1., filename = None, all_clusters = True):
+    if filename is None:
+        filename = test_equilateral.__name__
+    XX = [[scale, 0., 0.], [0., scale, 0], [0., 0., scale]]
+    XX = np.array(XX)
+    dr = DRUHG(algorithm='slow', max_ranking=200, limit1=1, limit2=1000, verbose=False)
+    dr.fit(XX)
+    # two clusters
+    # assert (len(dr.parents_) == 2)
+
+    labels = dr.labels_
+    print('pairs', dr.mst_)
+    print('labels', dr.labels_)
+    n_clusters = len(set(labels)) - int(-1 in labels)
+    print('n_clusters', n_clusters)
+
+    assert (all(x == labels[0] for x in labels))
+
+
+def test_scaled_equilateral(scale = 1., filename = None, all_clusters = True):
+    if filename is None:
+        filename = test_scaled_equilateral.__name__
+    for i in range(-5, 4):
+        test_equilateral(scale=10 ** i, filename=filename, all_clusters=True)
+
+def test_line(size=4, scale = 1., filename = None, all_clusters = False):
+    if filename is None:
+        filename = test_line.__name__
     XX = []
-    for i in range(0,100):
-        XX.append([0.,i])
+    for i in range(0,size):
+        XX.append([0.,i*scale])
     XX = np.array(XX)
 
     dr = DRUHG(max_ranking=50, limit1=1, limit2=len(XX), verbose=False)
@@ -157,13 +200,32 @@ def test_longline():
     print ('pairs', dr.mst_)
     print ('labels', dr.labels_)
     # assert (len(dr.parents_)==0)
+    if _plot_graph:
+        plt.close('all')
+        dr.minimum_spanning_tree_.plot(vary_line_width=None)
+        plt.savefig(filename+'.png')
     labels = dr.labels_
+
+    if all_clusters:
+        assert (all(x == labels[0] for x in labels))
+        return
+
     assert (not all(x == labels[0] for x in labels))
     assert (labels[0] == labels[len(labels)-1])
     assert (labels[1] == labels[len(labels)-2])
     assert (labels[0] != labels[1])
     # assert (0 == 1)
 
+def test_longline(filename=None):
+    if filename is None:
+        filename = test_longline.__name__
+    test_line(size=100, filename = filename)
+
+def test_fiveline(filename=None):
+    if filename is None:
+        filename = test_fiveline.__name__
+    test_line(size=5, filename = filename)
+
 # def test_hypersquare():
 #     XX = []
 #     size, scale = 6, 1.
@@ -192,14 +254,18 @@ def test_longline():
 #     assert (n_clusters == 5)
 #     assert (0==1)
 
-def test_square():
+def test_square(showplot=True, size = 10, scale = 5., filename=None):
+    if filename is None:
+        filename = test_square.__name__
     XX = []
-    size, scale = 10, 1
     for i in range(0, size):
         for j in range(0, size):
             XX.append([scale*i,scale*j])
+
     XX = np.array(XX)
-    dr = DRUHG(max_ranking=10, algorithm='slow', limit1=1, limit2=len(XX), verbose=False)
+    np.random.shuffle(XX)
+
+    dr = DRUHG(max_ranking=20, algorithm='slow', limit1=1, limit2=len(XX), verbose=False)
     dr.fit(XX)
     s = 2*len(XX) - 2
     print (dr.mst_)
@@ -221,89 +287,37 @@ def test_square():
     for i in range(0, len(un)):
         print('square', un[i], cn[i] )
 
-    if _plot_graph:
+    if showplot and _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot(vary_line_width = False)
-        plt.savefig('test_square.png')
-
-    assert (n_clusters >= 5)
+        plt.savefig(filename+'.png')
     labels = dr.labels_
-    assert (not all(x == labels[0] for x in labels))
-    assert (labels[1] != labels[size])
-    assert (labels[1] != labels[2*size - 1])
-    assert (labels[size] != labels[2 * size - 1])
-    assert (labels[size] != labels[2 * size - 1])
-    assert (labels[1] != labels[len(labels)-2])
-    assert (labels[int(size*size/2)] >= 0)
+    un, cn = np.unique(labels, return_counts=True)
+    print('uniques', un, cn)
 
+    sorteds = np.argsort(cn)
+    print('hello')
+    print('hello2', un[sorteds[0]],un[sorteds[1]])
     # assert (False)
-
-
-def test_scaled_square():
-    XX = []
-    size, scale = 10, 0.01
-    for i in range(0, size):
-        for j in range(0, size):
-            XX.append([scale*i,scale*j])
-    XX = np.array(XX)
-    dr = DRUHG(max_ranking=200, limit2 = len(XX), verbose=False)
-    dr.fit(XX)
-    # s = 2*len(XX) - 2
-    # print (dr.mst_)
-    # print (dr.mst_[s-1], dr.mst_[s-2], XX[dr.mst_[s-1]], XX[dr.mst_[s-2]])
-    labels = dr.labels_
-    print (labels)
+    dr.relabel(limit1=1, limit2=len(XX), exclude=[un[sorteds[0]], un[sorteds[1]]])
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_scaled_square.png')
-    n_clusters = len(set(labels)) - int(-1 in labels)
-    print ('n_clusters', n_clusters)
-    assert (n_clusters==1)
-
-def test_scaled_square2():
-    XX = []
-    size, scale = 10, 0.01
-    for i in range(0, size):
-        for j in range(0, size):
-            XX.append([scale*i,scale*j])
-    XX = np.array(XX)
-    dr = DRUHG(max_ranking=200, algorithm='slow', limit1=1, limit2=len(XX), verbose=False)
-    dr.fit(XX)
-    s = 2*len(XX) - 2
-    print (dr.mst_)
-    print (dr.mst_[s-1], dr.mst_[s-2], XX[dr.mst_[s-1]], XX[dr.mst_[s-2]])
-    labels = dr.labels_
-    n_clusters = len(set(labels)) - int(-1 in labels)
-    print('n_clusters', n_clusters)
-    print (dr.mst_)
-    # print (XX)
-    print (dr.labels_)
-    # assert (n_clusters==1)
-    # labels = dr.relabel(limit1=1, limit2=size*2)
-    un, cn = np.unique(labels, return_counts=True)
-    for i in range(0, len(un)):
-        print('square2', un[i], cn[i] )
-
-    n_clusters = len(set(labels)) - int(-1 in labels)
-    # print('n_clusters', n_clusters)
-    print ('pairs', dr.mst_)
-    print ('labels', dr.labels_)
-
-    if _plot_graph:
-        plt.close('all')
-        plt.figure(figsize=(25, 25))
-        dr.minimum_spanning_tree_.plot(vary_line_width=False)
-        plt.savefig('test_scaled_square2.png')
-
+        plt.savefig(filename+'2'+'.png')
     assert (n_clusters >= 5)
-    assert (not all(x == labels[0] for x in labels))
-    assert (labels[1] != labels[size])
-    assert (labels[1] != labels[2*size - 1])
-    assert (labels[size] != labels[2 * size - 1])
-    assert (labels[size] != labels[2 * size - 1])
-    assert (labels[1] != labels[len(labels)-2])
-    assert (labels[int(size*size/2)] >= 0)
+    assert (cn[sorteds[-1]] == (size-2)**2)
+    assert (un[sorteds[0]] == -1 and cn[sorteds[0]] == 4)
+    for i in range(1,5):
+        assert (cn[sorteds[i]] == size - 2)
+    # assert (False)
+
+def test_scaled_square(filename=None):
+    if filename is None:
+        filename = test_scaled_square.__name__
+    for i in range(-5, 4):
+        test_square(scale=10**i, filename=filename)
+
+
 
 def test_two_squares():
     XX = []
@@ -323,14 +337,22 @@ def test_two_squares():
     assert (n_clusters==2)
 
 
-def test_particles():
+def test_particles(filename=None):
+    if filename is None:
+        filename = test_particles.__name__
     XX = [[-0.51,1.5], [1.51,1.5]]
     for i in range(-3, 5):
         for j in range(-6, 1):
             XX.append([i,j])
     XX = np.array(XX)
-    dr = DRUHG(max_ranking=200, limit1=1, limit2=len(XX), verbose=False)
+    dr = DRUHG(max_ranking=200, algorithm='slow', limit1=1, limit2=len(XX), verbose=False)
     dr.fit(XX)
+
+    if _plot_graph:
+        plt.close('all')
+        dr.minimum_spanning_tree_.plot()
+        plt.savefig(filename+'.png')
+
     s = 2*len(XX) - 2
     print (dr.mst_)
     print (dr.labels_)
@@ -356,7 +378,9 @@ def test_bomb():
     assert (labs[0]==labs[1]==labs[2]==labs[3])
     assert (np.count_nonzero(labs == labs[0]) == 4)
 
-def test_t():
+def test_t(filename=None):
+    if filename is None:
+        filename = test_t.__name__
     XX = []
     for i in range(1, 10):
         XX.append([0.,i])
@@ -374,14 +398,16 @@ def test_t():
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_t.png')
+        plt.savefig(filename+'.png')
     # t-center is an outlier too
     assert (n_clusters == 3)
     assert (np.count_nonzero(labels == -1) == 4)
     # assert (False)
 
 #
-def test_cross():
+def test_cross(filename=None):
+    if filename is None:
+        filename = test_cross.__name__
     XX = []
     for j in range(-10, 10):
         XX.append([j,0.])
@@ -402,20 +428,22 @@ def test_cross():
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_cross.png')
+        plt.savefig(filename+'.png')
 
     assert (n_clusters == 4)
     assert (np.count_nonzero(labels == -1) == 5)
     # assert (False)
 
 
-def test_cube(showplot=True):
+def test_cube(size=5, scale=1., showplot=True, filename=None):
+    if filename is None:
+        filename = test_cube.__name__
+
     XX = []
-    size = 5
     for i in range(0, size):
         for j in range(0, size):
             for k in range(0, size):
-                XX.append([i,j,k])
+                XX.append([i*scale,j*scale,k*scale])
     XX = np.array(XX)
     np.random.shuffle(XX)
     print(XX)
@@ -435,6 +463,10 @@ def test_cube(showplot=True):
     print ('n_clusters', n_clusters, set(labels))
     # print ('labels', labels)
 
+    # sorteds = np.argsort(counts)
+    # dr.relabel(limit1=1, limit2=len(XX), exclude=[unique[sorteds[0]], unique[sorteds[1]]])
+    # labels = dr.labels_
+
     if showplot and _plot_graph:
         import seaborn as sns
 
@@ -467,18 +499,47 @@ def test_cube(showplot=True):
 
         color_map[0] = (0., 0., 0., 0.15)
         colors = [color_map[x] for x in labels]
-
+        s = [50 for x in labels]
         # ax = fig.add_subplot(111, projection='3d')
-        ax.scatter(XX[:, 0:1], XX[:, 1:2], XX[:, 2:3], c=colors)
+        ax.scatter(XX[:, 0:1], XX[:, 1:2], XX[:, 2:3], c=colors, s=s)
+
+        for i in range(0,len(labels)-1):
+            print('jajjajaja')
+            print(i, XX[dr.mst_[2*i]])
+            x1, y1, z1 = list(XX[dr.mst_[2*i]])
+            x2, y2, z2 = list(XX[dr.mst_[2 * i + 1]])
+
+            ax.plot([x1,x2], [y1,y2], [z1,z2], c=colors[dr.mst_[2*i]])
+
         plt.show()
-        plt.savefig('test_cube1.png')
+        plt.savefig(filename+'1'+'.png')
 
     if showplot and _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_cube.png')
+        plt.savefig(filename+'.png')
 
     assert (n_clusters == 1+6+12)
+    unique, counts = np.unique(labels, return_counts=True)
+
+    assert (-1 in unique)
+    i = np.where(unique == -1)[0][0]
+
+    assert(counts[i] == 8)
+    counts = np.delete(counts,[i])
+
+    counts = np.sort(counts)[::-1]
+    print(counts, unique)
+
+    assert ((size-2)**3==counts[0])
+
+    for i in range(1, 1+6):
+       assert (counts[i]==(size-2)**2)
+
+
+    for i in range(7, 1+6+12):
+        assert (counts[i] == size-2)
+
     # # assert (False)
     # # labels = dr.relabel(limit1=1)
     # labels = dr.relabel(limit1=1, limit2=len(XX))
@@ -491,10 +552,18 @@ def test_cube(showplot=True):
     # assert (0==1)
 
 def test_loop_cube():
-    k = 1000
+    k = 100
     while k!=0:
-        print('+++++++++++++PREVED+++++++++++++', k - 1000)
-        test_cube(False)
+        # print('+++++++++++++PREVED+++++++++++++', k - 1000)
+        test_cube(showplot=False)
+        # assert (False)
+        k-=1
+
+def test_loop_square():
+    k = 100
+    while k!=0:
+        # print('+++++++++++++PREVED+++++++++++++', k - 1000)
+        test_square(False)
         # assert (False)
         k-=1
 
@@ -508,7 +577,9 @@ def test_druhg_sparse():
     dr.fit(sparse_X)
     print ('sparse labels', dr.labels_)
 
-def test_druhg_distance_matrix():
+def test_druhg_distance_matrix(filename=None):
+    if filename is None:
+        filename = test_druhg_distance_matrix.__name__
     D = distance.squareform(distance.pdist(X))
     D /= np.max(D)
 
@@ -521,7 +592,7 @@ def test_druhg_distance_matrix():
         plt.close('all')
         dr = DRUHG(metric="precomputed").fit(D)
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_druhg_distance_matrix1.png')
+        plt.savefig(filename+'1'+'.png')
     assert(n_clusters==4)
 
     dr = DRUHG(metric="precomputed", limit1=5).fit(D)
@@ -532,7 +603,7 @@ def test_druhg_distance_matrix():
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_druhg_distance_matrix2.png')
+        plt.savefig(filename+'2'+'.png')
 
     assert(n_clusters==4)
 
@@ -547,9 +618,11 @@ def test_moons_and_blobs():
 
     assert (n_clusters == 4)
 #
-def test_hdbscan_clusterable_data():
+def test_hdbscan_clusterable_data(filename=None):
+    if filename is None:
+        filename = test_hdbscan_clusterable_data.__name__
     XX = np.load('druhg\\tests\\clusterable_data.npy')
-    dr = DRUHG(max_ranking=50, algorithm='slow', verbose=False)
+    dr = DRUHG(max_ranking=50, verbose=False)
     dr.fit(XX)
     labels = dr.labels_
     uniques, counts = np.unique(labels, True)
@@ -560,13 +633,15 @@ def test_hdbscan_clusterable_data():
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_hdbscan_clusterable_data.png')
+        plt.savefig(filename+'.png')
 
     assert (n_clusters==6)
 
-def test_three_blobs():
+def test_three_blobs(filename=None):
+    if filename is None:
+        filename = test_three_blobs.__name__
     XX = np.load('druhg\\tests\\three_blobs.npy')
-    dr = DRUHG(max_ranking=3550, algorithm='slow', verbose=False)
+    dr = DRUHG(max_ranking=3550, verbose=False)
     dr.fit(XX)
     labels = dr.labels_
     uniques, counts = np.unique(labels, True)
@@ -577,14 +652,16 @@ def test_three_blobs():
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_three_blobs.png')
+        plt.savefig(filename+'.png')
 
     assert (n_clusters==3)
 #
-def test_chameleon():
+def test_chameleon(is_reciprocal=1, filename=None):
+    if filename is None:
+        filename = test_chameleon.__name__
     XX = pd.read_csv('druhg\\tests\\chameleon.csv', sep='\t', header=None)
     XX = np.array(XX)
-    dr = DRUHG(algorithm='slow', max_ranking=4200, limit1 = 1, limit2=len(XX), verbose=False)
+    dr = DRUHG(max_ranking=4200, limit1 = 1, limit2=len(XX), is_reciprocal=is_reciprocal, verbose=False)
     dr.fit(XX)
     labels = dr.labels_
     # labels = dr.relabel(limit1=1)
@@ -599,15 +676,15 @@ def test_chameleon():
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_cham.png')
+        plt.savefig(filename+'1'+'.png')
 
-    dr = DRUHG( max_ranking=200, limit1 = 1, limit2=int(len(XX)/4), exclude=[], verbose=False)
+    dr = DRUHG( max_ranking=200, limit1 = 1, limit2=int(len(XX)/4), is_reciprocal=is_reciprocal, exclude=[], verbose=False)
     dr.fit(XX)
 
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_cham2.png')
+        plt.savefig(filename+'2'+'.png')
 
     values, counts = np.unique(dr.labels_, return_counts=True)
     for i, v in enumerate(values):
@@ -615,15 +692,29 @@ def test_chameleon():
             print (v, counts[i])
 
     exc = dr.labels_[3024]
-    dr.relabel(limit1=3, limit2=int(len(XX)/4), exclude=[exc])
+    dr.relabel(limit1=3, limit2=int(len(XX)/4), is_reciprocal=is_reciprocal, exclude=[exc])
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_cham3.png')
+        plt.savefig(filename+'3'+'.png')
+
+    exc = dr.labels_[3024]
+    dr.relabel(limit1=3, limit2=int(len(XX)/8), is_reciprocal=is_reciprocal, exclude=[exc])
+    if _plot_graph:
+        plt.close('all')
+        dr.minimum_spanning_tree_.plot()
+        plt.savefig(filename+'4'+'.png')
 
     assert (n_clusters==6)
 #
-def test_synthetic_outliers():
+def test_reciprocal_chameleon(filename = None):
+    if filename is None:
+        filename = test_reciprocal_chameleon.__name__
+    test_chameleon(is_reciprocal=0, filename=filename)
+
+def test_synthetic_outliers(filename=None):
+    if filename is None:
+        filename = test_synthetic_outliers.__name__
     XX = pd.read_csv('druhg\\tests\\synthetic.csv', sep=',')
     XX.drop(u'outlier', axis=1, inplace=True)
     XX = np.array(XX)
@@ -633,7 +724,7 @@ def test_synthetic_outliers():
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_synthetic_outliers.png')
+        plt.savefig(filename+'.png')
 
     # values, counts = np.unique(dr.labels_, return_counts=True)
     # for i, v in enumerate(values):
@@ -647,7 +738,9 @@ def test_synthetic_outliers():
     assert (n_clusters==6)
 
 
-def test_compound():
+def test_compound1(filename=None):
+    if filename is None:
+        filename = test_compound1.__name__
     XX = pd.read_csv('druhg/tests/Compound.csv', sep=',', header=None).drop(2, axis=1)
     XX = np.array(XX)
     dr = DRUHG(max_ranking=200, limit1=3, limit2=len(XX), verbose=False)
@@ -656,7 +749,7 @@ def test_compound():
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_compound1.png')
+        plt.savefig(filename+'1'+'.png')
 
     labels = dr.labels_
     # labels = dr.relabel(limit1=1)
@@ -672,18 +765,47 @@ def test_compound():
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_compound2.png')
+        plt.savefig(filename+'2'+'.png')
 
     exc2 = labels[398]
     dr.relabel(limit1=3, limit2=len(XX), exclude=[exc, exc2])
     if _plot_graph:
         plt.close('all')
         dr.minimum_spanning_tree_.plot()
-        plt.savefig('test_compound3.png')
+        plt.savefig(filename+'3'+'.png')
 
 
     assert (n_clusters==4)
 
+
+def test_compound_egg(filename=None):
+    if filename is None:
+        filename = test_compound_egg.__name__
+    XX = pd.read_csv('druhg/tests/Compound.csv', sep=',', header=None)
+    XX = XX[(XX[2]==6) | (XX[2]==5)]
+    XX = XX.drop(2, axis=1)
+    XX = np.array(XX)
+    dr = DRUHG(max_ranking=200, limit1=3, limit2=len(XX), verbose=False)
+    dr.fit(XX)
+
+    if _plot_graph:
+        plt.close('all')
+        dr.minimum_spanning_tree_.plot()
+        plt.savefig(filename+'.png')
+
+    labels = dr.labels_
+    # labels = dr.relabel(limit1=1)
+    n_clusters = len(set(labels)) - int(-1 in labels)
+    # np.save('labels_compound', labels)
+    print('n_clusters', n_clusters, set(labels))
+    # dr.relabel(limit1=3, limit2=len(XX), exclude=[exc])
+    # labels = dr.labels_
+    n_clusters2 = len(set(labels)) - int(-1 in labels)
+    print ( n_clusters2, set(labels))
+
+    assert (n_clusters==2)
+    # assert (False)
+
 def test_copycat():
     XX = [[0]]*100
     XX = np.array(XX)
@@ -743,196 +865,56 @@ def test_copycats3(): # should fail until weights are made
     n_clusters = len(set(labels)) - int(-1 in labels)
     assert (n_clusters==3)
 #
-# def test_cube2():
-#     XX = [[2,2,2],
-#         [0,1,1],
-#         [2,2,1],
-#         [3,3,1],
-#         [2,2,4],
-#         [1,4,1],
-#         [1,4,2],
-#         [3,2,4],
-#         [0,1,0],
-#         [2,3,0],
-#         [3,2,0],
-#         [2,2,3],
-#         [3,0,4],
-#         [3,2,3],
-#         [4,3,1],
-#         [1,2,4],
-#         [0,2,0],
-#         [4,4,1],
-#         [1,3,1],
-#         [0,4,2],
-#         [2,1,0],
-#         [3,2,2],
-#         [1,3,2],
-#         [0,2,3],
-#         [4,2,2],
-#         [0,4,4],
-#         [2,0,0],
-#         [1,1,0],
-#         [2,1,2],
-#         [0,1,4],
-#         [4,4,0],
-#         [0,4,3],
-#         [2,1,4],
-#         [1,1,1],
-#         [0,4,0],
-#         [3,0,1],
-#         [3,1,2],
-#         [0,3,3],
-#         [4,4,4],
-#         [0,2,4],
-#         [2,0,1],
-#         [1,0,0],
-#         [0,1,3],
-#         [2,4,2],
-#         [4,4,3],
-#         [1,1,2],
-#         [2,1,1],
-#         [0,0,2],
-#         [3,0,3],
-#         [4,3,0],
-#         [2,1,3],
-#         [4,0,3],
-#         [3,0,0],
-#         [1,0,4],
-#         [0,0,3],
-#         [3,1,3],
-#         [0,0,1],
-#         [3,4,1],
-#         [1,4,3],
-#         [2,3,4],
-#         [3,3,0],
-#         [4,1,3],
-#         [3,3,3],
-#         [1,4,4],
-#         [4,1,4],
-#         [2,4,1],
-#         [3,4,2],
-#         [0,2,2],
-#         [4,0,1],
-#         [3,3,4],
-#         [2,4,3],
-#         [2,3,2],
-#         [3,1,0],
-#         [4,0,2],
-#         [3,4,0],
-#         [4,3,4],
-#         [3,2,1],
-#         [4,2,4],
-#         [3,0,2],
-#         [4,1,0],
-#         [2,0,3],
-#         [4,2,0],
-#         [4,0,4],
-#         [3,3,2],
-#         [0,0,0],
-#         [2,0,2],
-#         [1,2,0],
-#         [2,2,0],
-#         [1,2,2],
-#         [1,2,3],
-#         [3,4,3],
-#         [2,4,4],
-#         [0,0,4],
-#         [0,3,1],
-#         [1,1,4],
-#         [3,1,1],
-#         [4,1,2],
-#         [0,4,1],
-#         [1,0,1],
-#         [4,3,2],
-#         [3,1,4],
-#         [4,4,2],
-#         [4,3,3],
-#         [1,3,4],
-#         [4,1,1],
-#         [0,2,1],
-#         [2,4,0],
-#         [2,3,1],
-#         [3,4,4],
-#         [1,3,0],
-#         [2,3,3],
-#         [0,3,4],
-#         [4,2,1],
-#         [0,3,0],
-#         [1,4,0],
-#         [1,1,3],
-#         [0,3,2],
-#         [0,1,2],
-#         [4,2,3],
-#         [1,3,3],
-#         [2,0,4],
-#         [4,0,0],
-#         [1,0,3],
-#         [1,0,2],
-#         [1,2,1]]
-#     XX = np.array(XX)
-#     # for i, x in enumerate(XX):
-#     #     print (i, x)
-#     dr = DRUHG(algorithm='slow', max_ranking=200, limit2=int(len(XX)), verbose=False)
-#     dr.fit(XX)
-#     s = 2*len(XX) - 2
-#     print (dr.mst_)
-#     # assert (0==1)
-#     labels = dr.labels_
-#     unique, counts = np.unique(labels, return_counts=True)
-#     print (unique, counts)
-#     labels = dr.relabel(limit1=1, limit2=len(XX)/2)
+# def test_speed0():
+#     XX = []
+#     size = 5
+#     for i in range(0, size):
+#         for j in range(0, size):
+#             for k in range(0, size):
+#                 XX.append([i, j, k])
 #
-#     n_clusters = len(set(labels)) - int(-1 in labels)
-#     print ('n_clusters', n_clusters, set(labels))
-#     # print ('labels', labels)
+#     XX = pd.read_csv('druhg\\tests\\synthetic.csv', sep=',')
+#     XX.drop(u'outlier', axis=1, inplace=True)
 #
-#     if _plot_graph:
-#         import seaborn as sns
+#     return np.array(XX)
 #
-#         plt.close('all')
-#         fig = plt.figure()
-#         ax = Axes3D(fig)
 #
-#         unique, counts = np.unique(labels, return_counts=True)
-#         sorteds = np.argsort(counts)
-#         s = len(sorteds)
-#
-#         i = sorteds[s - 1]
-#         max_size = counts[i]
-#         if unique[i] == 0:
-#             max_size = counts[sorteds[s - 2]]
-#
-#         color_map = {}
-#         palette = sns.color_palette('bright', s + 1)
-#         col = 0
-#         a = (1. - 0.3) / (max_size - 1)
-#         b = 0.3 - a
-#         while s:
-#             s -= 1
-#             i = sorteds[s]
-#             if unique[i] == 0:
-#                 continue
-#             alpha = a * counts[i] + b
-#             color_map[unique[i]] = palette[col] + (alpha,)
-#             col += 1
-#
-#         color_map[0] = (0., 0., 0., 0.15)
-#         colors = [color_map[x] for x in labels]
-#
-#         # ax = fig.add_subplot(111, projection='3d')
-#         ax.scatter(XX[:, 0:1], XX[:, 1:2], XX[:, 2:3], c=colors)
-#         plt.show()
-#         plt.savefig('test_cube1.png')
-#
-#     if _plot_graph:
-#         plt.close('all')
-#         dr.minimum_spanning_tree_.plot()
-#         plt.savefig('test_cube.png')
-#
-#     assert (n_clusters == 1+6+12)
-
-# def test_loop_cube2():
-#     k = 1000
+# def test_speed1(k=100):
+#     XX = test_speed0()
 #     while k!=0:
-#         test_cube2()
-#         k-=1
\ No newline at end of file
+#         k -= 1
+#         np.random.shuffle(XX)
+#         dr = DRUHG(algorithm='fast', max_ranking=200, limit1=1, limit2=int(len(XX) / 2), verbose=False)
+#         dr.fit(XX)
+#
+# def test_speed2(k=100):
+#     XX = test_speed0()
+#     while k!=0:
+#         k -= 1
+#         np.random.shuffle(XX)
+#         dr = DRUHG(algorithm='fastminus', max_ranking=200, limit1=1, limit2=int(len(XX) / 2), verbose=False)
+#         dr.fit(XX)
+#
+# def test_speed3(k=100):
+#     XX = test_speed0()
+#     while k!=0:
+#         k -= 1
+#         np.random.shuffle(XX)
+#         dr = DRUHG(algorithm='slow+', max_ranking=200, limit1=1, limit2=int(len(XX) / 2), verbose=False)
+#         dr.fit(XX)
+#
+# def test_speed4(k=100):
+#     XX = test_speed0()
+#     while k!=0:
+#         k -= 1
+#         np.random.shuffle(XX)
+#         dr = DRUHG(algorithm='slow', max_ranking=200, limit1=1, limit2=int(len(XX) / 2), verbose=False)
+#         dr.fit(XX)
+#
+# def test_speed5(k=100):
+#     XX = test_speed0()
+#     while k!=0:
+#         k -= 1
+#         np.random.shuffle(XX)
+#         dr = DRUHG(algorithm='slow', max_ranking=200, limit1=1, limit2=int(len(XX) / 2), verbose=False)
+#         dr.fit(XX)
\ No newline at end of file
